var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var TimerTargetImpl_1;
import '@nativescript/macos-node-api';
const timeoutCallbacks = new Map();
let TimerTargetImpl = class TimerTargetImpl extends NSObject {
    static { TimerTargetImpl_1 = this; }
    callback;
    disposed;
    id;
    shouldRepeat;
    static initWithCallback(callback, id, shouldRepeat) {
        const handler = TimerTargetImpl_1.new();
        handler.callback = callback;
        handler.id = id;
        handler.shouldRepeat = shouldRepeat;
        return handler;
    }
    tick(_timer) {
        if (!this.disposed) {
            this.callback();
        }
        if (!this.shouldRepeat) {
            this.unregister();
        }
    }
    unregister() {
        if (!this.disposed) {
            this.disposed = true;
            const timer = timeoutCallbacks.get(this.id)?.k;
            timer?.invalidate();
            timeoutCallbacks.delete(this.id);
        }
    }
    static ObjCExposedMethods = {
        tick: { returns: interop.types.void, params: [NSTimer] },
    };
};
TimerTargetImpl = TimerTargetImpl_1 = __decorate([
    NativeClass
], TimerTargetImpl);
let timerId = 0;
function createTimerAndGetId(callback, milliseconds, shouldRepeat) {
    // Cast to Number
    milliseconds += 0;
    timerId++;
    const id = timerId;
    const timerTarget = TimerTargetImpl.initWithCallback(callback, id, shouldRepeat);
    const timer = NSTimer.timerWithTimeIntervalTargetSelectorUserInfoRepeats(milliseconds / 1000, timerTarget, 'tick', null, shouldRepeat);
    // https://github.com/NativeScript/NativeScript/issues/2116
    NSRunLoop.currentRunLoop.addTimerForMode(timer, NSRunLoopCommonModes);
    const pair = {
        k: timer,
        v: timerTarget,
    };
    timeoutCallbacks.set(id, pair);
    return id;
}
export function setTimeout(callback, milliseconds = 0, ...args) {
    const invoke = () => callback(...args);
    return createTimerAndGetId(invoke, milliseconds, false);
}
export function clearTimeout(id) {
    const pair = timeoutCallbacks.get(id);
    if (pair && pair.v) {
        pair.v.unregister();
    }
}
export function setInterval(callback, milliseconds = 0, ...args) {
    const invoke = () => callback(...args);
    return createTimerAndGetId(invoke, milliseconds, true);
}
//# sourceMappingURL=timer.js.map