var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var OutlineViewDataSource_1;
import '@nativescript/macos-node-api';
import { Event } from '../../dom/dom-utils.js';
import { native } from '../decorators/native.js';
import { view } from '../decorators/view.js';
import { View } from '../view/view.js';
export class OutlineViewItemSelectedEvent extends Event {
    constructor(index, item, eventDict) {
        super('itemSelected', eventDict);
        this.index = index;
        this.item = item;
    }
}
let OutlineViewDataSource = class OutlineViewDataSource extends NSObject {
    static { OutlineViewDataSource_1 = this; }
    static ObjCProtocols = [NSOutlineViewDataSource, NSOutlineViewDelegate];
    static initWithOwner(owner) {
        const dataSource = OutlineViewDataSource_1.new();
        dataSource._owner = owner;
        return dataSource;
    }
    _owner;
    get outline() {
        return this._owner?.deref();
    }
    outlineViewNumberOfChildrenOfItem(_outlineView, item) {
        if (item) {
            return item.children.length - 2;
        }
        else {
            return this.outline.children.length;
        }
    }
    outlineViewViewForTableColumnItem(_outlineView, _tableColumn, item) {
        return item.nativeView;
    }
    outlineViewIsItemExpandable(_outlineView, item) {
        return item.children.length > 2;
    }
    outlineViewChildOfItem(_outlineView, index, item) {
        if (item) {
            return item.children[index + 2];
        }
        else {
            return this.outline.children[index];
        }
    }
    outlineViewObjectValueForTableColumnByItem(_outlineView, _tableColumn, item) {
        return item;
    }
    outlineViewSelectionDidChange(notification) {
        const outlineView = notification.object;
        const owner = this.outline;
        if (owner && outlineView) {
            const item = outlineView.itemAtRow(outlineView.selectedRow);
            owner.dispatchEvent(new OutlineViewItemSelectedEvent(outlineView.selectedRow, item));
            if (item) {
                item.dispatchSelectedEvent();
            }
        }
    }
};
OutlineViewDataSource = OutlineViewDataSource_1 = __decorate([
    NativeClass
], OutlineViewDataSource);
let Outline = class Outline extends View {
    nativeView = undefined;
    dataSource;
    initNativeView() {
        const outline = NSOutlineView.new();
        this.dataSource = OutlineViewDataSource.initWithOwner(new WeakRef(this));
        // @ts-expect-error It's nullable
        outline.headerView = null;
        outline.indentationPerLevel = 10;
        outline.allowsColumnReordering = false;
        outline.allowsColumnResizing = false;
        outline.allowsColumnSelection = false;
        outline.allowsEmptySelection = false;
        outline.rowSizeStyle = NSTableViewRowSizeStyle.Medium;
        outline.delegate = this.dataSource;
        outline.dataSource = this.dataSource;
        const tableColumn = NSTableColumn.alloc().initWithIdentifier('Column');
        outline.addTableColumn(tableColumn);
        outline.outlineTableColumn = tableColumn;
        this.nativeView = outline;
        return this.nativeView;
    }
    addNativeChild(_child) {
        this.nativeView?.reloadData();
        this.nativeView?.expandItem(_child);
        if (this.__pendingCellSelection) {
            this.selectCell(this.__pendingCellSelection);
            this.__pendingCellSelection = null;
        }
    }
    connectedCallback() {
        super.connectedCallback();
    }
    removeNativeChild(_child) {
        this.nativeView?.reloadData();
    }
    __pendingCellSelection = null;
    expandParentsOfItem(item) {
        while (item !== null && this.nativeView) {
            const parent = this.nativeView.parentForItem(item);
            if (!this.nativeView.isExpandable(parent)) {
                break;
            }
            if (!this.nativeView.isItemExpanded(parent)) {
                this.nativeView.expandItem(parent);
            }
            item = parent;
        }
    }
    selectCell(cell) {
        if (cell) {
            this.__pendingCellSelection = cell;
            if (cell.nativeView) {
                let index = this.nativeView?.rowForItem(cell) || 0;
                if (index < 0) {
                    this.expandParentsOfItem(cell);
                    index = this.nativeView?.rowForItem(cell) || 0;
                    if (index < 0)
                        return;
                }
                this.nativeView?.selectRowIndexesByExtendingSelection(NSIndexSet.indexSetWithIndex(index), false);
                this.__pendingCellSelection = null;
            }
        }
        else {
            this.nativeView?.deselectAll(null);
        }
    }
};
__decorate([
    native({
        setNative(view, _key, value) {
            if (view.nativeView) {
                view.nativeView.indentationPerLevel = value;
            }
        },
    }),
    __metadata("design:type", Number)
], Outline.prototype, "identationPerLevel", void 0);
__decorate([
    native({
        setNative(view, _key, value) {
            if (view.nativeView) {
                view.nativeView.rowSizeStyle = value;
            }
        },
    }),
    __metadata("design:type", Object)
], Outline.prototype, "rowSizeStyle", void 0);
Outline = __decorate([
    view({
        name: 'HTMLOutlineElement',
        tagName: 'outline',
    })
], Outline);
export { Outline };
//# sourceMappingURL=outline.js.map