var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var WebViewDelegate_1;
import '@nativescript/macos-node-api';
import { Event } from '../../dom/dom-utils.js';
import { native } from '../decorators/native.js';
import { view } from '../decorators/view.js';
import { View } from '../view/view.js';
objc.import('WebKit');
export class WebviewNavigationEvent extends Event {
    constructor(type, url, navigationType, eventDict) {
        super(type, eventDict);
        this.url = url;
        this.navigationType = navigationType;
    }
}
export class LoadStartedEvent extends WebviewNavigationEvent {
    constructor(url, navigationType, eventDict) {
        super('loadStarted', url, navigationType, eventDict);
    }
}
export class LoadFinishedEvent extends WebviewNavigationEvent {
    constructor(url, eventDict) {
        super('loadFinished', url, undefined, eventDict);
    }
}
let WebViewDelegate = class WebViewDelegate extends NSObject {
    static { WebViewDelegate_1 = this; }
    static ObjCProtocols = [WKUIDelegate, WKNavigationDelegate];
    static initWithOwner(owner) {
        const delegate = WebViewDelegate_1.new();
        delegate._owner = owner;
        return delegate;
    }
    webViewCreateWebViewWithConfigurationForNavigationActionWindowFeatures(webView, _configuration, navigationAction, _windowFeatures) {
        if (navigationAction && (!navigationAction.targetFrame || (navigationAction.targetFrame && !navigationAction.targetFrame.isMainFrame))) {
            webView.loadRequest(navigationAction.request);
        }
        return webView;
    }
    webViewDecidePolicyForNavigationActionDecisionHandler(_webView, navigationAction, decisionHandler) {
        const owner = this._owner?.deref();
        if (owner && navigationAction.request.URL) {
            let navType = 'other';
            switch (navigationAction.navigationType) {
                case WKNavigationType.LinkActivated:
                    navType = 'linkClicked';
                    break;
                case WKNavigationType.FormSubmitted:
                    navType = 'formSubmitted';
                    break;
                case WKNavigationType.BackForward:
                    navType = 'backForward';
                    break;
                case WKNavigationType.Reload:
                    navType = 'reload';
                    break;
                case WKNavigationType.FormResubmitted:
                    navType = 'formResubmitted';
                    break;
            }
            decisionHandler(WKNavigationActionPolicy.Allow);
            owner.dispatchEvent(new LoadStartedEvent(navigationAction.request.URL.absoluteString, navType));
        }
    }
    webViewDidFinishNavigation(webView, _navigation) {
        const owner = this._owner?.deref();
        if (owner) {
            const src = owner.src;
            if (webView.URL) {
                owner.dispatchEvent(new LoadFinishedEvent(src));
            }
        }
    }
};
WebViewDelegate = WebViewDelegate_1 = __decorate([
    NativeClass
], WebViewDelegate);
let WebView = class WebView extends View {
    get isLeaf() {
        return true;
    }
    delegate;
    nativeView = undefined;
    initNativeView() {
        const config = WKWebViewConfiguration.new();
        this.nativeView = WKWebView.alloc().initWithFrameConfiguration(CGRectZero, config);
        this.nativeView.setValueForKey(false, 'drawsBackground');
        this.delegate = WebViewDelegate.initWithOwner(new WeakRef(this));
        this.nativeView.UIDelegate = this.delegate;
        this.nativeView.navigationDelegate = this.delegate;
        this.nativeView.configuration.preferences.setValueForKey(true, 'allowFileAccessFromFileURLs');
        return this.nativeView;
    }
    applyLayout(parentLayout) {
        super.applyLayout(parentLayout);
        if (this.nativeView) {
            this.nativeView.translatesAutoresizingMaskIntoConstraints = true;
        }
    }
    loadURL(value) {
        if (this.nativeView) {
            const url = typeof value === 'string' ? value : value.href;
            const nsUrl = NSURL.URLWithString(url);
            if (url?.indexOf('http') > -1) {
                this.nativeView.loadRequest(NSURLRequest.requestWithURL(nsUrl));
            }
            else {
                this.nativeView.loadFileURLAllowingReadAccessToURL(nsUrl, nsUrl.URLByDeletingLastPathComponent);
            }
        }
    }
    executeJavaScript(src) {
        this.nativeView?.evaluateJavaScriptCompletionHandler(src, (result, error) => {
            if (error) {
                console.error(error);
            }
        });
    }
};
__decorate([
    native({
        setNative(view, _key, value) {
            view.loadURL(value);
        },
    }),
    __metadata("design:type", Object)
], WebView.prototype, "src", void 0);
__decorate([
    native({
        setNative(view, _key, value) {
            if (view?.nativeView) {
                view.nativeView.isInspectable = value;
            }
        },
    }),
    __metadata("design:type", Boolean)
], WebView.prototype, "debug", void 0);
WebView = __decorate([
    view({
        name: 'HTMLWebviewElement',
        tagName: 'webview',
    })
], WebView);
export { WebView };
//# sourceMappingURL=webview.js.map