var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Color } from '../../style/utils/color.js';
import { native } from '../decorators/native.js';
import { overrides } from '../decorators/overrides.js';
import { view } from '../decorators/view.js';
import { ViewBase } from '../view/view-base.js';
import { MainWindowController, NativeWindow } from './native-window.js';
let Window = class Window extends ViewBase {
    isRoot = true;
    shouldAttachToParentNativeView = false;
    initNativeView() {
        this.nativeView = NativeWindow.new();
        this.mainWindowCtrl = MainWindowController.initWithOwner(new WeakRef(this));
        this.mainWindowCtrl.window = this.nativeView;
        this.nativeView.windowController = this.mainWindowCtrl;
        this.nativeView.isReleasedWhenClosed = false;
        this.nativeView.contentViewController = NSViewController.new();
        this.viewController = this.nativeView.contentViewController;
        // This view will become the main window if no other window is present;
        if (!NativeScriptApplication.window) {
            NativeScriptApplication.window = this;
            NativeScriptApplication.showMainWindow = () => {
                this.nativeView?.becomeMainWindow();
                this.nativeView?.displayIfNeeded();
                this.nativeView?.makeKeyAndOrderFront(NSApp);
            };
        }
        this.nativeView.owner = new WeakRef(this);
        // The toolbar style is best set to .automatic
        // But it appears to go as .unifiedCompact if
        // you set as .automatic and titleVisibility as
        // .hidden
        this.nativeView.toolbarStyle = NSWindowToolbarStyle.Automatic;
        // Set this property to .hidden to gain more toolbar space
        this.nativeView.titleVisibility = NSWindowTitleVisibility.Visible;
        this.nativeView.delegate = this.nativeView;
        return this.nativeView;
    }
    disposeNativeView() {
        if (this.nativeView) {
            this.nativeView = undefined;
        }
    }
    applyLayout(parentLayout) {
        if (parentLayout)
            return;
        const layout = this.yogaNode.getComputedLayout();
        if (this.nativeView) {
            const screenSize = NSScreen.mainScreen.frame.size;
            const frame = {
                origin: {
                    x: screenSize.width / 2 - layout.width / 2,
                    y: screenSize.height / 2 - layout.height / 2,
                },
                size: {
                    width: layout.width,
                    height: layout.height,
                },
            };
            this.nativeView?.setFrameDisplay(frame, false);
        }
        this.applyLayoutToChildern(layout);
    }
    open() {
        if (NativeScriptApplication.window === this) {
            this.nativeView?.becomeMainWindow();
            this.nativeView?.displayIfNeeded();
            this.nativeView?.makeKeyAndOrderFront(NSApp);
        }
        else {
            const window = this.parentNode.nativeView?.window;
            if (window && window?.windowController && this.nativeView) {
                const frameCentered = {
                    x: window.frame.origin.x + window.frame.size.width / 2 - this.nativeView.frame.size.width / 2,
                    y: window.frame.origin.y + window.frame.size.height / 2 - this.nativeView.frame.size.height / 2,
                };
                this.nativeView.setFrameOrigin(frameCentered);
                window?.windowController.showWindow(this.nativeView);
                this.nativeView.makeKeyAndOrderFront(NSApp);
            }
        }
    }
    _isModal;
    openAsModal(relativeToWindow) {
        this._isModal = true;
        if (relativeToWindow) {
            const window = this.parentNode.nativeView?.window;
            if (window) {
                NSApp.runModalForWindowRelativeToWindow(this.nativeView, window);
            }
        }
        else {
            NSApp.runModalForWindow(this.nativeView);
        }
    }
    closeModalWindow() {
        this.close();
    }
    close() {
        this.nativeView?.close();
        if (this.nativeView?.isKeyWindow) {
            this.nativeView?.resignKeyWindow();
        }
    }
    insertBefore(node, child) {
        // if (!this.firstChild) {
        super.insertBefore(node, child);
        // } else {
        //   throw new Error("Window can only have one child");
        // }
        return node;
    }
    addNativeChild(child) {
        if (this.nativeView) {
            if (child.nodeName === 'TOOLBAR') {
                this.nativeView.toolbar = child.nativeView;
            }
            else {
                this.nativeView.contentViewController.view.addSubview(child.nativeView);
            }
        }
    }
    removeNativeChild(child) {
        if (this.nativeView) {
            if (child.nodeName === 'TOOLBAR') {
                // @ts-expect-error it can be null
                this.nativeView.toolbar = null;
            }
            else {
                child.nativeView.removeFromSuperview();
            }
        }
    }
    onSetBackgroundColor(_key, value) {
        if (this.nativeView) {
            this.nativeView.backgroundColor = new Color(value).toNSColor();
        }
    }
};
__decorate([
    native({
        setNative(view, _key, value) {
            if (view.nativeView) {
                view.nativeView.titlebarAppearsTransparent = value;
            }
        },
    }),
    __metadata("design:type", Boolean)
], Window.prototype, "transparentTitleBar", void 0);
__decorate([
    native({
        setNative(view, _key, value) {
            if (view.nativeView) {
                view.nativeView.title = value;
            }
        },
    }),
    __metadata("design:type", String)
], Window.prototype, "title", void 0);
__decorate([
    native({
        setNative(view, _key, value) {
            if (view.nativeView) {
                view.nativeView.subtitle = value;
            }
        },
    }),
    __metadata("design:type", String)
], Window.prototype, "subtitle", void 0);
__decorate([
    overrides('backgroundColor'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", void 0)
], Window.prototype, "onSetBackgroundColor", null);
__decorate([
    native({
        setNative(view, _key, value) {
            if (view.nativeView) {
                view.nativeView.styleMask = value;
            }
        },
    }),
    __metadata("design:type", Object)
], Window.prototype, "styleMask", void 0);
__decorate([
    native({
        setNative(view, _key, value) {
            if (view.nativeView) {
                view.nativeView.isReleasedWhenClosed = value;
            }
        },
    }),
    __metadata("design:type", Boolean)
], Window.prototype, "releasedWhenClosed", void 0);
Window = __decorate([
    view({
        name: 'HTMLWindowElement',
        tagName: 'window',
    })
], Window);
export { Window };
//# sourceMappingURL=window.js.map